@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef COMMON_CAIRO_H
@@ -31,25 +31,35 @@ static PangoFontDescription * get_font_from_theme () {
 	return rv;
 }
 
+static float host_fg_color[4] = { .9, .9, .9, 1.0 };
+static float host_bg_color[4] = { .24, .24, .24, 1.0 };
+
+static void set_host_color (int which, uint32_t color) {
+	switch(which) {
+		case 0:
+			host_fg_color[0] = ((color >> 24) & 0xff) / 255.0;
+			host_fg_color[1] = ((color >> 16) & 0xff) / 255.0;
+			host_fg_color[2] = ((color >>  8) & 0xff) / 255.0;
+			host_fg_color[3] = ((color >>  0) & 0xff) / 255.0;
+			break;
+		case 1:
+			host_bg_color[0] = ((color >> 24) & 0xff) / 255.0;
+			host_bg_color[1] = ((color >> 16) & 0xff) / 255.0;
+			host_bg_color[2] = ((color >>  8) & 0xff) / 255.0;
+			host_bg_color[3] = ((color >>  0) & 0xff) / 255.0;
+			break;
+		default:
+			break;
+	}
+}
+
 static void get_color_from_theme (int which, float *col) {
 	switch(which) {
 		default: // fg
-			col[0] = .9;
-			col[1] = .9;
-			col[2] = .9;
-			col[3] = 1.0;
+			memcpy (col, host_fg_color, 4 * sizeof (float));
 			break;
 		case 1: // bg
-			col[0] = 61/255.0;
-			col[1] = 61/255.0;
-			col[2] = 61/255.0;
-			col[3] = 1.0;
-			break;
-		case 2: // fg alt
-			col[0] = 0;
-			col[1] = 0;
-			col[2] = 0;
-			col[3] = 1.0;
+			memcpy (col, host_bg_color, 4 * sizeof (float));
 			break;
 	}
 }
